function [f_es,x_es,G]=BSURE(y,smp,doPlot)
% function [f_es,x_es,grid_num,G]=BSURE(y,m,smp)
% The BSURE algorithm
% Input:
% y - the data vector
% smp - vector containing the time stamps of the samples
% doPlot - 1 shows the resulting estimates at each iteration, otherwiswe
% not.
% Output:
% f_es - frequency estimates
% x_es - amplitude estimates


m = length(y);
re_do = 1;
mu =1/sum_square_abs(y);
re_doSum = 0;
f_new = [];
f_old = 1;

while re_do && re_doSum < 20
    re_do = 0;
    grid_num = 15;
    G = grid_num;
    
    f_grid = linspace(.1,.3,grid_num)';
    L = floor(1./f_grid);
    fgrid = [];
    nonZeroWithinGroup = ones(length(L),L(1));
    for ell = 1:length(L)
        fgridtemp = f_grid(ell)*[1:L(ell)];
        fgrid = [fgrid; fgridtemp'];
        nonZeroWithinGroup(ell,L(ell)+1:end) = 0;
    end
    f_grid = fgrid;
    grid_num = length(f_grid);
    x_new=ones(grid_num,1);
    x_old=zeros(grid_num,1);
    f=f_grid;
    
    threshold=0.05;
    index_amp=1:grid_num;
    epsilon = 1;
    max_iter=100;
    index_group = 1:G;
    lambda = .1;
    
    
    
    for n_itr=1:max_iter
        normSaveX = norm(x_old-x_new);
        if length(f_new)==length(f_old)
            normSave = mean(abs(sort(f_new)-sort(f_old)));
        else
            normSave = inf;
        end
        
        if normSave<1e-8 && normSaveX<1e-3
            break;
        end
        if epsilon>1e-8 && norm(x_old-x_new)<epsilon^0.5
            epsilon=epsilon/10;
        end
        x_old=x_new;
        
        dd=1./(abs(x_old).^2+epsilon);
        D=diag(dd);
        G = length(index_group);
        C = zeros(size(D));
        Lsum = cumsum(L);
        flag = 0;
        for g = 1:G
            if g == 1
                ind = 1:L(1);
            else
                ind = Lsum(g-1)+1:Lsum(g);
            end
            Di = 1./(norm(x_old(ind))^2+epsilon);
            if flag
                C(ind,ind) = diag(sqrt(L(g))*Di*mu*ones(L(g),1))/(sum(abs(x_old(ind))>threshold)+epsilon)/sqrt(norm(x_old(ind(1)))+epsilon)/(sum(abs(x_old(ind))>threshold)+epsilon);
            else
                C(ind,ind) = diag(sqrt(L(g))*Di*mu*ones(L(g),1))/(1e-5)/(sum(abs(x_old(ind))>threshold)+epsilon);
            end
        end
        
        
        if flag~=1
            A=exp(-2i*pi*smp'*f');
            lambda = max( 5*(norm(y-A*x_old,'fro')^2 )/m,1e-8);
        end
        
        IND = [];
        index_groupIND = [];
        % pruning and lambda update
        if epsilon<1e-1
            index_group = 1:G;
            flag = 1;
            for g = 1:G
                if g == 1
                    ind = 1:L(1);
                else
                    ind = Lsum(g-1)+1:Lsum(g);
                end
                ind0 = find(abs(x_old(ind))< threshold);
                x_old(ind(ind0)) = 0;
                tempInd = find(nonZeroWithinGroup(g,:));
                tempInd = tempInd(ind0);
                nonZeroWithinGroup(g,tempInd) = 0;
                
                
                X = x_old(ind);
                index_Tgroup = norms(X,2)>threshold;
                
                if (index_Tgroup) == 0
                    
                    IND = [IND; ind(:)];
                    index_groupIND = [index_groupIND; g];
                end
                
            end
            totalIndex = 1:length(x_old);
            totalIndex(IND) = [];
            f(IND) = [];
            x_old(IND) = [];
            index_amp = totalIndex;
            L(index_groupIND) = [];
            index_group(index_groupIND) = [];
            nonZeroWithinGroup(index_groupIND,:) = [];
            Lsum = cumsum(L);
            G = length(index_group);
            temp = [];
            ftemp = [];
            for g=1:G
                if g == 1
                    ind = 1:L(1);
                else
                    ind = Lsum(g-1)+1:Lsum(g);
                end
                temp2 = x_old(ind);
                ftemp2 = f(ind);
                ftemp2(abs(temp2)<threshold) =[];
                index_amp(abs(temp2)<threshold) = [];
                temp2(abs(temp2)<threshold) = [];
                nonZeroWithinGroup(g,abs(temp2)<threshold) = 0;
                temp = [temp; temp2];
                ftemp = [ftemp; ftemp2];
            end
            L = sum(nonZeroWithinGroup,2);
            Lsum = cumsum(L);
            x_old=  temp;
            f = ftemp;
            D=D(index_amp,index_amp);
            
            if isempty(f) || n_itr==max_iter
                re_do = 1;
                re_doSum = re_doSum + re_do;
                mu=mu/2;
                f=nan;
                x_new = nan;
                break
            end
                     
            C = zeros(size(D));
            for g = 1:G
                
                if g == 1
                    ind = 1:L(1);
                else
                    ind = Lsum(g-1)+1:Lsum(g);
                end
               
                
                
                Di = 1./(norm(x_old(ind))^2+epsilon);
                if flag
                    C(ind,ind) = diag(sqrt(L(g))*Di*mu*ones(L(g),1))/(sum(abs(x_old(ind))>threshold)+epsilon)/sqrt(norm(x_old(ind(1)))+epsilon)/(sum(abs(x_old(ind))>threshold)+epsilon);
                else
                    C(ind,ind) = diag(sqrt(L(g))*Di*mu*ones(L(g),1))/(1e-5)/(sum(abs(x_old(ind))>threshold)+epsilon);
                end
            end
        end
        f_old = f;
        f_new=f;
        G = length(index_group);
        
        
        
        
        %% sequential update of dictionary parameters
        
        for i=1:G
            A=exp(-2i*pi*smp'*f');
            if i == 1
                ind = 1:L(1);
            else
                ind = Lsum(i-1)+1:Lsum(i);
            end
            Lvector = find(nonZeroWithinGroup(i,:));
            A_d=-2i*pi*exp(-2i*pi*smp'*f');
            dA_theta=zeros(m,length(f));
            dA_theta(:,ind)=bsxfun(@times, smp',A_d(:,ind));
            dA_theta(:,ind) = dA_theta(:,ind)*diag(Lvector);
            notInd = 1:length(index_amp);
            notInd(ind) = [];
            
      
            if length(ind)==1 && isempty(notInd)
                re_do = 1;
                mu = mu/2;
                
                break
            end
          
            yS =y;

            ddtt=-inv(lambda*(D+C)+A'*A)*(dA_theta'*A+A'*dA_theta)*inv(lambda*(D+C)+A'*A);
            d_obj_theta_s = trace( -(yS*yS')*( dA_theta*inv(lambda*(D+C)+A'*A)*A'+A*ddtt*A'+A*inv(lambda*(D+C)+A'*A)*dA_theta' ) );
            der=sign(real(d_obj_theta_s));
            
            stepsize= 1/2/grid_num;
            f_new(ind)=f(ind)-stepsize*der.*Lvector';
            f_new=mod(f_new,1);
            A_new=exp(-2i*pi*smp'*f_new');
            func_val=-y'*A_new*inv(lambda*(D+C)+A_new'*A_new)*A_new'*y;
            sur_val=-y'*A*inv(lambda*(D+C)+A'*A)*A'*y;
            
            maxit=0;
            while func_val>sur_val && maxit<50
                stepsize=1/2*stepsize;
                f_new(ind)=f(ind)-stepsize*der.*Lvector';
                f_new=mod(f_new,1);
                A_new=exp(-2i*pi*smp'*f_new');
                func_val=-y'*A_new*inv(lambda*(D+C)+A_new'*A_new)*A_new'*y;
                maxit=maxit+1;
            end
            
            

            
            f=f_new;
        end
        A=exp(-2i*pi*smp'*f');
        x_new=(lambda*(D+C)+A'*A)\A'*y;
        lambda= lambda*.5;
        if doPlot
            figure(1)
            stem(f,abs(x_new))
            title(num2str(n_itr))
            drawnow
        end
    end
end
f_es=f;
ind0 = abs(x_new)<threshold;
x_new(ind0) = [];
f_es(ind0) = [];
x_es=x_new;

end



